@@ -2,21 +2,21 @@ setMethod("pfmix",
           signature(x = "REBMIX"),
 function(x,
   Dataset,
-  pos, 
-  variables, 
-  lower.tail, 
+  pos,
+  variables,
+  lower.tail,
   log.p, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
-  
+
   if (missing(x)) {
     stop(sQuote("x"), " object of class REBMIX is requested!", call. = FALSE)
   }
-  
+
   if (missing(Dataset)) {
     stop(sQuote("Dataset"), " must not be empty!", call. = FALSE)
   }
-  
+
   if (!is.data.frame(Dataset)) {
     stop(sQuote("Dataset"), " data frame is requested!", call. = FALSE)
   }
@@ -31,18 +31,18 @@ function(x,
 
   if (n < 1) {
     stop(sQuote("Dataset"), " number of rows in data frame must be greater than 1!", call. = FALSE)
-  }  
+  }
 
   if (!is.wholenumber(pos)) {
     stop(sQuote("pos"), " integer is requested!", call. = FALSE)
   }
-  
+
   length(pos) <- 1
 
   if ((pos < 1) || (pos > nrow(x@summary))) {
     stop(sQuote("pos"), " must be greater than 0 and less or equal than ", nrow(x@summary), "!", call. = FALSE)
   }
-  
+
   if (length(variables) != 0) {
     if (!is.wholenumber(variables)) {
       stop(sQuote("variables"), " integer is requested!", call. = FALSE)
@@ -51,40 +51,40 @@ function(x,
     if ((min(variables) < 1) || (max(variables) > d)) {
       stop(sQuote("variables"), " must be greater than 0 and less or equal than ", d, "!", call. = FALSE)
     }
-    
-    variables <- unique(variables)   
+
+    variables <- unique(variables)
   }
   else {
     variables <- 1:d
   }
-  
+
   if (!is.logical(lower.tail)) {
     stop(sQuote("lower.tail"), " logical is requested!", call. = FALSE)
   }
-  
+
   if (!is.logical(log.p)) {
     stop(sQuote("log.p"), " logical is requested!", call. = FALSE)
   }
-  
+
   w <- x@w[[pos]]
-  
+
   c <- length(w)
-  
+
   Theta <- x@Theta[[pos]]
-  
+
   Names <- names(Theta)
-  
+
   pdf <- Theta[grep("pdf", Names)]
 
   theta1 <- Theta[grep("theta1", Names)]
-  
-  theta2 <- Theta[grep("theta2", Names)]  
+
+  theta2 <- Theta[grep("theta2", Names)]
 
   f <- array(data = 0.0, dim = n, dimnames = NULL)
 
   for (i in 1:c) {
     fi <- rep(1.0, n)
-    
+
     for (j in variables) {
       if (pdf[[i]][j] == .rebmix$pdf[1]) {
         fi <- fi * pnorm(as.numeric(Dataset[, j]), mean = as.numeric(theta1[[i]][j]), sd = as.numeric(theta2[[i]][j]), ...)
@@ -122,22 +122,27 @@ function(x,
           Kappa = as.double(theta2[[i]][j]),
           F = double(n),
           PACKAGE = "rebmix")
-          
+
         fi <- fi * output$F
-      }       
+      }
     }
-    
+
     f <- f + as.numeric(w[i]) * fi
   }
-  
+
   output <- as.data.frame(cbind(Dataset[, variables], f), stringsAsFactors = FALSE)
 
-  colnames(output) <- c(paste("x", if (d > 1) variables else "", sep = ""), "F")      
-  
+  if (is.null(colnames(Dataset))) {
+    colnames(output) <- c(paste("x", if (d > 1) variables else "", sep = ""), "F")
+  }
+  else {
+    colnames(output) <- c(colnames(Dataset)[variables], "F")
+  }
+
   options(digits = digits)
 
   rm(list = ls()[!(ls() %in% c("output"))])
-  
+
   output
 }) ## pfmix
 
@@ -145,21 +150,21 @@ setMethod("pfmix",
           signature(x = "REBMVNORM"),
 function(x,
   Dataset,
-  pos, 
-  variables, 
-  lower.tail, 
+  pos,
+  variables,
+  lower.tail,
   log.p, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
-  
+
   if (missing(x)) {
     stop(sQuote("x"), " object of class REBMVNORM is requested!", call. = FALSE)
   }
-  
+
   if (missing(Dataset)) {
     stop(sQuote("Dataset"), " must not be empty!", call. = FALSE)
   }
-  
+
   if (!is.data.frame(Dataset)) {
     stop(sQuote("Dataset"), " data frame is requested!", call. = FALSE)
   }
@@ -174,18 +179,18 @@ function(x,
 
   if (n < 1) {
     stop(sQuote("Dataset"), " number of rows in data frame must be greater than 1!", call. = FALSE)
-  }  
+  }
 
   if (!is.wholenumber(pos)) {
     stop(sQuote("pos"), " integer is requested!", call. = FALSE)
   }
-  
+
   length(pos) <- 1
 
   if ((pos < 1) || (pos > nrow(x@summary))) {
     stop(sQuote("pos"), " must be greater than 0 and less or equal than ", nrow(x@summary), "!", call. = FALSE)
   }
-  
+
   if (length(variables) != 0) {
     if (!is.wholenumber(variables)) {
       stop(sQuote("variables"), " integer is requested!", call. = FALSE)
@@ -194,63 +199,68 @@ function(x,
     if ((min(variables) < 1) || (max(variables) > d)) {
       stop(sQuote("variables"), " must be greater than 0 and less or equal than ", d, "!", call. = FALSE)
     }
-    
-    variables <- unique(variables)   
+
+    variables <- unique(variables)
   }
   else {
     variables <- 1:d
   }
-  
+
   if (!is.logical(lower.tail)) {
     stop(sQuote("lower.tail"), " logical is requested!", call. = FALSE)
   }
-  
+
   if (!is.logical(log.p)) {
     stop(sQuote("log.p"), " logical is requested!", call. = FALSE)
   }
-  
+
   w <- x@w[[pos]]
-  
+
   c <- length(w)
-  
+
   Theta <- x@Theta[[pos]]
-  
+
   Names <- names(Theta)
-  
+
   pdf <- Theta[grep("pdf", Names)]
 
   theta1 <- Theta[grep("theta1", Names)]
-  
-  theta2 <- Theta[grep("theta2", Names)]  
+
+  theta2 <- Theta[grep("theta2", Names)]
 
   f <- array(data = 0.0, dim = n, dimnames = NULL)
 
   for (i in 1:c) {
     fi <- rep(0.0, n)
-    
+
     if (all(pdf[[i]] == .rebmix$pdf[1])) {
       mean <- as.numeric(theta1[[i]][variables])
-    
+
       sigma <- matrix(theta2[[i]], ncol = d, byrow = TRUE)
-      
+
       sigma <- sigma[variables, variables]
-      
+
       for (j in 1:n) {
         fi[j] <- pmvnorm(upper = as.numeric(Dataset[j, variables]), mean = mean, sigma = sigma, ...)
       }
     }
-    
+
     f <- f + as.numeric(w[i]) * fi
   }
-  
+
   output <- as.data.frame(cbind(Dataset[, variables], f), stringsAsFactors = FALSE)
 
-  colnames(output) <- c(paste("x", if (d > 1) variables else "", sep = ""), "F")      
-  
+  if (is.null(colnames(Dataset))) {
+    colnames(output) <- c(paste("x", if (d > 1) variables else "", sep = ""), "F")
+  }
+  else {
+    colnames(output) <- c(colnames(Dataset)[variables], "F")
+  }
+
   options(digits = digits)
 
   rm(list = ls()[!(ls() %in% c("output"))])
-  
+
   output
 }) ## pfmix
 
