@@ -1,4 +1,4 @@
-setMethod("plot", 
+setMethod("plot",
           signature(x = "RCLSMIX", y = "missing"),
 function(x,
   y,
@@ -34,21 +34,21 @@ function(x,
   }
 
   d <- ncol(x@Dataset)
-  
+
   Zp <- as.numeric(levels(x@Zp))[x@Zp]
   Zt <- as.numeric(levels(x@Zt))[x@Zt]
-  
-  zlim <- c(0, max(1, max(Zp) - 1)); zmax <- zlim[2]   
-  
+
+  zlim <- c(0, max(1, max(Zp) - 1)); zmax <- zlim[2]
+
   unique.Zp <- unique(Zp); s <- length(unique.Zp); sort.unique.Zp <- sort(unique.Zp)
-  
+
   nrow <- max(1, nrow)
   ncol <- max(1, ncol)
 
   N <- d * (d - 1) / 2
-  
-  opar <- list(); ipar <- 1  
-  
+
+  opar <- list(); ipar <- 1
+
   par(mfrow = c(nrow, ncol),
     cex = cex,
     cex.axis = 1.0,
@@ -63,23 +63,23 @@ function(x,
   par(oma = c(1 + 0.2, 0.2, 0.2, 0.2))
 
   ey <- as.matrix(x@Dataset); ep <- Zp - 1
-  
+
   error <- as.integer(Zt != Zp)
-  
+
   ramp <- colorRamp(colors = c("magenta", "blue", "cyan", "green", "yellow", "red"),
     space = "rgb",
     interpolate = "linear")
-    
+
   plot.col <- rgb(ramp(ep / zmax), maxColorValue = 255)
-  
+
   legend.col <- rgb(ramp((sort.unique.Zp - 1) / zmax), maxColorValue = 255)
-  
+
   legend.text <- as.character(sort.unique.Zp)
-  
+
   legend.pch <- rep(plot.pch, s)
-  
+
   which.error <- which(error == 1); which.not.error <- which(error != 1)
-  
+
   if (N > 0) {
     figno <- 0
 
@@ -99,7 +99,7 @@ function(x,
           lwd = 1,
           cex = plot.cex,
           pch = plot.pch)
-          
+
         points(x = ey[which.error, i],
           y = ey[which.error, j],
           type = "p",
@@ -108,9 +108,9 @@ function(x,
           xlab = "",
           ylab = "",
           col = "black",
-          lwd = 1,
-          cex = plot.cex,
-          pch = 1)          
+          lwd = 0.6,
+          cex = 0.6 * plot.cex,
+          pch = 1)
 
         box(col = fg, lty = "solid", lwd = 1)
 
@@ -142,19 +142,19 @@ function(x,
           adj = 0.5,
           padj = 0.2,
           cex = cex)
-          
+
         figno <- figno + 1
 
         if ((figno == nrow * ncol) || ((i == d - 1) && (j == d))) {
-          par(fig = c(0, 1, 0, 1), 
-            oma = c(0, 0, 0, 0), 
-            mar = c(0, 0, 0, 0), 
+          par(fig = c(0, 1, 0, 1),
+            oma = c(0, 0, 0, 0),
+            mar = c(0, 0, 0, 0),
             new = TRUE)
-          
+
           plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
-          
+
           .legendA(s = s, text = legend.text, col = legend.col, pch = legend.pch, error = sum(error) != 0)
-  
+
           par(mfrow = c(nrow, ncol),
             cex = cex,
             cex.axis = 1.0,
@@ -165,13 +165,13 @@ function(x,
             oma = c(1.2, 0.2, 0.2, 0.2),
             pty = pty,
             tcl = tcl, ...)
-    
+
           par(oma = c(1 + 0.2, 0.2, 0.2, 0.2))
 
           figno <- 0
         }
-        
-        opar[[ipar]] <- par(no.readonly = TRUE); ipar <- ipar + 1            
+
+        opar[[ipar]] <- par(no.readonly = TRUE); ipar <- ipar + 1
       }
     }
   }
@@ -184,13 +184,13 @@ function(x,
       xlab = "",
       ylab = "",
       xlim = range(ey[, 1]),
-      ylim = range(Zp),      
+      ylim = range(Zp),
       col = plot.col[which.not.error],
       axes = FALSE,
       lwd = 1,
       cex = plot.cex,
       pch = plot.pch)
-          
+
     points(x = ey[which.error, 1],
       y = Zp[which.error],
       type = "p",
@@ -199,9 +199,9 @@ function(x,
       xlab = "",
       ylab = "",
       col = "black",
-      lwd = 1,
-      cex = plot.cex,
-      pch = 1)       
+      lwd = 0.6,
+      cex = 0.6 * plot.cex,
+      pch = 1)
 
     box(col = fg, lty = "solid", lwd = 1)
 
@@ -234,15 +234,15 @@ function(x,
       padj = 0.2,
       cex = cex)
 
-    par(fig = c(0, 1, 0, 1), 
-      oma = c(0, 0, 0, 0), 
-      mar = c(0, 0, 0, 0), 
+    par(fig = c(0, 1, 0, 1),
+      oma = c(0, 0, 0, 0),
+      mar = c(0, 0, 0, 0),
       new = TRUE)
-          
+
     plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
-          
+
    .legendA(s = s, text = legend.text, col = legend.col, pch = legend.pch, error = sum(error) != 0)
-      
+
     par(mfrow = c(nrow, ncol),
       cex = cex,
       cex.axis = 1.0,
@@ -253,18 +253,18 @@ function(x,
       oma = c(1.2, 0.2, 0.2, 0.2),
       pty = pty,
       tcl = tcl, ...)
-      
+
     par(oma = c(1 + 0.2, 0.2, 0.2, 0.2))
-    
-    opar[[ipar]] <- par(no.readonly = TRUE); ipar <- ipar + 1        
+
+    opar[[ipar]] <- par(no.readonly = TRUE); ipar <- ipar + 1
   }
-  
+
   rm(list = ls()[!(ls() %in% c("opar"))])
 
   invisible(opar)
-}) # plot         
+}) # plot
 
-setMethod("plot", 
+setMethod("plot",
           signature(x = "RCLSMVNORM", y = "missing"),
 function(x,
   y,
@@ -300,21 +300,21 @@ function(x,
   }
 
   d <- ncol(x@Dataset)
-  
+
   Zp <- as.numeric(levels(x@Zp))[x@Zp]
   Zt <- as.numeric(levels(x@Zt))[x@Zt]
-  
-  zlim <- c(0, max(1, max(Zp) - 1)); zmax <- zlim[2]   
-  
+
+  zlim <- c(0, max(1, max(Zp) - 1)); zmax <- zlim[2]
+
   unique.Zp <- unique(Zp); s <- length(unique.Zp); sort.unique.Zp <- sort(unique.Zp)
-  
+
   nrow <- max(1, nrow)
   ncol <- max(1, ncol)
 
   N <- d * (d - 1) / 2
-  
-  opar <- list(); ipar <- 1  
-  
+
+  opar <- list(); ipar <- 1
+
   par(mfrow = c(nrow, ncol),
     cex = cex,
     cex.axis = 1.0,
@@ -329,23 +329,23 @@ function(x,
   par(oma = c(1 + 0.2, 0.2, 0.2, 0.2))
 
   ey <- as.matrix(x@Dataset); ep <- Zp - 1
-  
+
   error <- as.integer(Zt != Zp)
-  
+
   ramp <- colorRamp(colors = c("magenta", "blue", "cyan", "green", "yellow", "red"),
     space = "rgb",
     interpolate = "linear")
-    
+
   plot.col <- rgb(ramp(ep / zmax), maxColorValue = 255)
-  
+
   legend.col <- rgb(ramp((sort.unique.Zp - 1) / zmax), maxColorValue = 255)
-  
+
   legend.text <- as.character(sort.unique.Zp)
-  
+
   legend.pch <- rep(plot.pch, s)
-  
+
   which.error <- which(error == 1); which.not.error <- which(error != 1)
-  
+
   if (N > 0) {
     figno <- 0
 
@@ -365,7 +365,7 @@ function(x,
           lwd = 1,
           cex = plot.cex,
           pch = plot.pch)
-          
+
         points(x = ey[which.error, i],
           y = ey[which.error, j],
           type = "p",
@@ -374,9 +374,9 @@ function(x,
           xlab = "",
           ylab = "",
           col = "black",
-          lwd = 1,
-          cex = plot.cex,
-          pch = 1)          
+          lwd = 0.6,
+          cex = 0.6 * plot.cex,
+          pch = 1)
 
         box(col = fg, lty = "solid", lwd = 1)
 
@@ -408,19 +408,19 @@ function(x,
           adj = 0.5,
           padj = 0.2,
           cex = cex)
-          
+
         figno <- figno + 1
 
         if ((figno == nrow * ncol) || ((i == d - 1) && (j == d))) {
-          par(fig = c(0, 1, 0, 1), 
-            oma = c(0, 0, 0, 0), 
-            mar = c(0, 0, 0, 0), 
+          par(fig = c(0, 1, 0, 1),
+            oma = c(0, 0, 0, 0),
+            mar = c(0, 0, 0, 0),
             new = TRUE)
-          
+
           plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
-          
+
           .legendA(s = s, text = legend.text, col = legend.col, pch = legend.pch, error = sum(error) != 0)
-  
+
           par(mfrow = c(nrow, ncol),
             cex = cex,
             cex.axis = 1.0,
@@ -431,13 +431,13 @@ function(x,
             oma = c(1.2, 0.2, 0.2, 0.2),
             pty = pty,
             tcl = tcl, ...)
-    
+
           par(oma = c(1 + 0.2, 0.2, 0.2, 0.2))
 
           figno <- 0
         }
-        
-        opar[[ipar]] <- par(no.readonly = TRUE); ipar <- ipar + 1            
+
+        opar[[ipar]] <- par(no.readonly = TRUE); ipar <- ipar + 1
       }
     }
   }
@@ -450,13 +450,13 @@ function(x,
       xlab = "",
       ylab = "",
       xlim = range(ey[, 1]),
-      ylim = range(Zp),      
+      ylim = range(Zp),
       col = plot.col[which.not.error],
       axes = FALSE,
       lwd = 1,
       cex = plot.cex,
       pch = plot.pch)
-          
+
     points(x = ey[which.error, 1],
       y = Zp[which.error],
       type = "p",
@@ -465,9 +465,9 @@ function(x,
       xlab = "",
       ylab = "",
       col = "black",
-      lwd = 1,
-      cex = plot.cex,
-      pch = 1)       
+      lwd = 0.6,
+      cex = 0.6 * plot.cex,
+      pch = 1)
 
     box(col = fg, lty = "solid", lwd = 1)
 
@@ -500,15 +500,15 @@ function(x,
       padj = 0.2,
       cex = cex)
 
-    par(fig = c(0, 1, 0, 1), 
-      oma = c(0, 0, 0, 0), 
-      mar = c(0, 0, 0, 0), 
+    par(fig = c(0, 1, 0, 1),
+      oma = c(0, 0, 0, 0),
+      mar = c(0, 0, 0, 0),
       new = TRUE)
-          
+
     plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
-          
+
    .legendA(s = s, text = legend.text, col = legend.col, pch = legend.pch, error = sum(error) != 0)
-      
+
     par(mfrow = c(nrow, ncol),
       cex = cex,
       cex.axis = 1.0,
@@ -519,12 +519,12 @@ function(x,
       oma = c(1.2, 0.2, 0.2, 0.2),
       pty = pty,
       tcl = tcl, ...)
-      
+
     par(oma = c(1 + 0.2, 0.2, 0.2, 0.2))
-    
-    opar[[ipar]] <- par(no.readonly = TRUE); ipar <- ipar + 1        
+
+    opar[[ipar]] <- par(no.readonly = TRUE); ipar <- ipar + 1
   }
-  
+
   rm(list = ls()[!(ls() %in% c("opar"))])
 
   invisible(opar)
